---------------------------------------------------------------------
--== Import ===--
---------------------------------------------------------------------

import "Turbine.Gameplay"
import "Turbine.UI"
import "Turbine.UI.Lotro"
import "Souru.LootLogs.Type"
import "Souru.LootLogs.Class"
import "Souru.LootLogs.InstanceList"

---------------------------------------------------------------------
--==  ===--
---------------------------------------------------------------------

me = Turbine.Gameplay.LocalPlayer:GetInstance()
name = me:GetName()

FONTHEADING = Turbine.UI.Lotro.Font.BookAntiquaBold19
FONT = Turbine.UI.Lotro.Font.Verdana16

---------------------------------------------------------------------
--== Basic Functions ===--
---------------------------------------------------------------------

function SaveLogs()

	Turbine.PluginData.Save(Turbine.DataScope.Account, "LootLogs", LootList)

end

function SaveSettings()

	Turbine.PluginData.Save(Turbine.DataScope.Character, "LootSettings", LootSettings)

end

function CheckForMe()

	for i,v in ipairs(LootList) do 
		
		if v["player"] == name then
			return i
		end
		
	end
	
	return false

end

function AddMe()

	local index = table.getn(LootList)+1
	
	LootList[index] = {}
	LootList[index]["player"] = name
	LootList[index]["loot"] = deepcopy(InstancesSave)
	
	currentIndex = index
	
	SaveLogs()

end

function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

function CheckForReset()

	local currentTime = Turbine.Engine.GetGameTime()+200

	for i,players in ipairs(LootList) do 
	
		local instanceCount = table.getn(players["loot"])
		
		if instanceCount ~= table.getn(InstancesSave) then
		
			for n=instanceCount, table.getn(InstancesSave), 1 do
				
				players["loot"][n] = deepcopy(InstancesSave[n])
			
			end
		
		end
		
		for j,inis in ipairs(players["loot"]) do 

			local isAnyChestLocked = false
			
			local chestCount = table.getn(inis)
			
			if chestCount ~= table.getn(InstancesSave[j]) then
			
				players["loot"][j] = deepcopy(InstancesSave[j])
			
			end

			for k,chests in ipairs(inis) do 
	
				local tierCount = table.getn(chests)
				
				if tierCount ~= table.getn(InstancesSave[j][1]) then
				
					players["loot"][j] = deepcopy(InstancesSave[j])
				
				end
	
				for m,tiers in ipairs(chests) do 

					if currentTime > tiers["nextReset"]   then
						if tiers["nextReset"] ~= 0 then
							tiers["nextReset"]  = 0
							tiers["amount"]  = "-"
						end
					else	
						isAnyChestLocked = true
					end
	
				end
	
			end

			inis["show"] = isAnyChestLocked

		end

	end

	SaveLogs()

end

function showWindow()

	base = UI.LogWindow()

end

function showIcon()

	icon:Show()

end

---------------------------------------------------------------------
--== LoadIn ===--
---------------------------------------------------------------------

if Turbine.PluginData.Load(Turbine.DataScope.Account, "LootLogs") ~= nil then

	LootList = Turbine.PluginData.Load(Turbine.DataScope.Account, "LootLogs")
	
	currentIndex = CheckForMe()
	
	if currentIndex == false then
		AddMe()
	end

else

	LootList = {}
	AddMe()
	
	SaveLogs()
end

if Turbine.PluginData.Load(Turbine.DataScope.Character, "LootSettings") ~= nil then

	LootSettings = Turbine.PluginData.Load(Turbine.DataScope.Character, "LootSettings")

else

	LootSettings = {}

	LootSettings["window"] = {}
	LootSettings["window"]["left"] = 500
	LootSettings["window"]["top"] = 500
	LootSettings["window"]["width"] = 850
	LootSettings["window"]["height"] = 500
	LootSettings["icon"] = {}
	LootSettings["icon"]["left"] = 200
	LootSettings["icon"]["top"] = 200
	
end

---------------------------------------------------------------------
--== Window ===--
---------------------------------------------------------------------

import "Souru.LootLogs.UI"
icon = UI.WindowIcon()

---------------------------------------------------------------------
--== Loot Functions ===--
---------------------------------------------------------------------

function CalcNextReset(ini,boss,tier)

	local dateTime = Turbine.Engine.GetDate()
	local timeTillReset = 0
	local nextResetDayIndex
	
	if(dateTime["Hour"] < 9) then
	
		timeTillReset = (540-(dateTime["Hour"]*60))+(dateTime["Minute"])
		nextResetDayIndex = dateTime["DayOfWeek"]
	
	else
	
		timeTillReset = (540+(1440-((dateTime["Hour"]*60)+(60-dateTime["Minute"]))))
		
		if(dateTime["DayOfWeek"] < 7) then
			nextResetDayIndex = dateTime["DayOfWeek"] +1
		else
			nextResetDayIndex = 1
		end
	
	end
	
	local bool = false
	local counter = 0
	while(bool == false) do
	
		if (Instances[ini][boss][tier]["reset"][nextResetDayIndex] == true) then
			bool = true
		else
			counter = counter + 1
			if(nextResetDayIndex < 7) then
				nextResetDayIndex = nextResetDayIndex+1
			else
				nextResetDayIndex = 1
			end
		end
		
	end
		
	timeTillReset = (timeTillReset + (counter*1440))*60	
	LootList[currentIndex]["loot"][ini][boss][tier]["nextReset"] = Turbine.Engine.GetGameTime()+timeTillReset
	
end

function FindChest(text, remaining)

	local nameStart
	local nameEnd

	for i,inis in ipairs(Instances) do 
	
		for j,bosses in ipairs(inis) do 
		
			nameStart, nameEnd = string.find(text, bosses["name"])
			
			if nameStart ~= nil then
			
				local chestName = string.sub(text, nameEnd+1)

				for k,tier in ipairs(bosses) do 
				
					if chestName == tier["text"] then

						if LootList[currentIndex]["loot"][i] == nil then					
							LootList[currentIndex]["loot"][i] = InstancesSave[i]
						end
						
						LootList[currentIndex]["loot"][i][j][k]["amount"] = remaining
							LootList[currentIndex]["loot"][i]["show"] = true
							CalcNextReset(i,j,k)
							SaveLogs()
							
							if base ~= nil then
								base:UpdateContent()
							end		
					
					end
					
				end
				
			end
		
		end
	
	end

end


function Turbine.Chat.Received(sender, args)
	
	if args.ChatType == Turbine.ChatType.Standard then

		if (string.find(args.Message, ": You have (%d+) favoured completion")) ~= nil then -- FAVORED

			FindChest(string.sub(args.Message, 0, string.find(args.Message, ": You have (%d+) favoured completion")-1), tonumber(string.match(string.sub(args.Message, string.find(args.Message, ": You have (%d+) favoured completion")) , "%d+")))

		elseif (string.find(args.Message, ": You have (%d+) completion")) ~= nil then -- COMON
		
			FindChest(string.sub(args.Message, 0, string.find(args.Message, ": You have (%d+) completion")-1), tonumber(string.match(string.sub(args.Message, string.find(args.Message, ": You have (%d+) completion")) , "%d+"))*(-1))

		elseif (string.find(args.Message, " resets in: ")) ~= nil then -- RESET
		
			FindChest(string.sub(args.Message, 0, string.find(args.Message, "resets in: ")-2), 0)
		end
		
	end
	
end